﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;

namespace Demo
{
    [StructLayout(LayoutKind.Sequential)]
    public struct PlateBox
    {
        public float xmin;   // 左上角x轴坐标
        public float ymin;   // 左上角y轴坐标
        public float xmax;   // 右下角x轴坐标
        public float ymax;   // 右下角y轴坐标
        public float score;  // 车牌框置信度
    };

    [StructLayout(LayoutKind.Sequential)]
    public struct PlateBasicInfo
    {
        public int label;                                       // 识别的目标类型，小于等于1是车牌，大于1其他目标
        public float prob;                                      // 车牌识别置信度
        public PlateBox bbox;                                    // 车牌位置
        [MarshalAs(UnmanagedType.ByValArray, ArraySubType = UnmanagedType.I4, SizeConst = 12)]
        public float[] confidences;                              // 车牌号码单字置信度数组
        public int len;                                          // 车牌号码单字置信度数组长度
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 64)]
        public byte[] plateNos_;                                 // UTF8变长码编码的车牌号码。
        public int plateNoBytes;                                 // 车牌号码实际实际长度，不超过MAX_PLATE_NO_BYTES变量定义
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 16)]
        public String plateColor;

 
        public IntPtr imageBuffer;                  // 图片Buffer
        public int imageBufferSize;

        public string plateNos
        {
            get
            {
                if ( plateNoBytes > 0)
                { 
                    return Encoding.UTF8.GetString(plateNos_, 0, plateNoBytes);
                }
				else {
					return "";
				} 
            }
        }
    };
    public enum API_STATUS
    {
        OK,
        FAILED
    };

    static class LLPRAPI
    {


        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void Callback(ref PlateBasicInfo pbi);  

        // please change to the exact DLL file name
        const string DllName = "mlpr-api-1.0.1-7220.dll";

        
        [DllImport(DllName, CallingConvention = CallingConvention.Cdecl, EntryPoint = "init_client_api2")]
        public extern static API_STATUS initClientApi(String deviceIP, String localIP, int localPort, [MarshalAs(UnmanagedType.FunctionPtr)]Callback callback);

        [DllImport(DllName, CallingConvention = CallingConvention.Cdecl, EntryPoint = "close_client_api")]
        public extern static API_STATUS closeClientApi();

    }
}
