/*
 * demo.cpp
 *
 *  Created on: Sep 7, 2020
 *  all right reserved by Xinhuan Intelligence co., ltd
 *  contact us at https://github.com/lqian/light-lpr
 *
 */
#include <thread>
#include <iostream>
#include <fstream>
#include <sstream>
#include <stdio.h>
#include <stdlib.h>
#include <chrono>
#include <time.h>

#include "dev_client_api.h"

using namespace std;

void demo_callback_func(PlateBasicInfo * pbi) {
	string plateNo(pbi->plateNos, pbi->plateNoBytes);
	printf("%16s plate no: %s at [%f, %f, %f, %f] with confidence: %f, plate type: %d \n",
			pbi->deviceIP,
			plateNo.c_str(),
			pbi->bbox.xmin, pbi->bbox.ymin, pbi->bbox.xmax, pbi->bbox.ymax,
			pbi->prob, pbi->plateType);

	if (pbi->speed.limitation > 0 || pbi->speed.value > 0) {
		printf("speed meter: limitation: %d ,value: %d, status: %d \n",
				pbi->speed.limitation, pbi->speed.value, pbi->speed.status );
	}

	if (pbi->plateImageBufferSize > 0)
	{
		printf("plate image size: %d\n", pbi->plateImageBufferSize);
	}

	if (pbi->direction >=0 && pbi->direction <= 3) {
		int dx = pbi->direction & 1;
		int dy = (pbi->direction & 2) >> 1;
		printf("direction x: %d, y: %d\n", dx, dy);
	}

	if (pbi->imageBuffer == NULL ) return;

	time_t curr=time(NULL);
	stringstream ss;
	ss << curr << ".jpg";
	//save image to file
	ofstream out(ss.str().c_str(), ios_base::binary);
	if (out.is_open()) {
		out.write(pbi->imageBuffer, pbi->imageBufferSize);
		out.flush();
		out.close();
		cout << "save file success!" << endl;
	}
	else {
		cout << "open file error!" << endl;
	}
}

int main(int argc, char ** argv) {
	if (argc < 3) {
		printf("usage: DEMO [local_ip] [local_port] [device ip]...\n"
				"example: DEMO 192.168.1.102 20790 192.168.1.20 192.168.1.201 \n");
		exit(1);
	}
	char * local_ip = argv[1];
	int local_port = atoi(argv[2]);
	API_STATUS ret = init_client(local_ip, local_port, demo_callback_func);

	if (ret  != API_STATUS::OK) {
		printf("init client api failed!\n");
		exit(1);
	}


	for (int i = 3; i < argc; i++) {
		if (connect_device(argv[i]) == API_STATUS::OK) {
			printf("connect device success: %s \n", argv[i]);
		}
		else {
			printf("connect device failed: %s \n", argv[i]);
		}
	}
	printf("--------------- devices connected ------------\n");

	std::this_thread::sleep_for(std::chrono::seconds(60)); // sleep 60 seconds

	close_client();  // the function can call in another thread

	exit(0);

}

int main2(int argc, char ** argv)
{
	int local_port = 12093;
	API_STATUS ret = API_STATUS::FAILED;

	while (ret != API_STATUS::OK)
	{
		ret = init_local_client(local_port, demo_callback_func);
		if (ret != API_STATUS::OK)
		{
			printf("try to connect after 3 seconds!\n");
		}
		std::this_thread::sleep_for(std::chrono::seconds(3));
	}

	printf("--------------- devices connected ------------\n");

	while (getchar() != 'q')
	{
		std::this_thread::sleep_for(std::chrono::seconds(60));
		printf("press key q for quit!");
	}

	close_client();  // the function can call in another thread

	exit(0);
}

