/*
 * mgt_demo.cpp
 *
 *  Created on: Sep 7, 2020
 *  all right reserved by Xinhuan Intelligence co., ltd
 *  contact us at https://github.com/lqian/light-lpr
 *
 */

#include <thread>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <chrono>
#include <string.h>


#include "dev_client_api.h"

int main(int argc, char ** argv) {
	if (argc <2 ) {
		printf("usage: mgt_demo -reboot [device ip] for reboot device\n"
				"mgt_demo -change_password [device_ip] [cur_pwd] [new_pwd] for change password\n"
				"mgt_demo -snapshot [device_ip] for take a snapshot from device\n"
				"mgt_demo -set_anchorbox [device_ip] [xmin] [ymin] [xmax] [ymax] set anchor box for device\n"
				"mgt_demo -get_anchorbox [device_ip] get anchor box for device\n"
				"mgt_demo -deviceinfo [device_ip] for basic device info\n"
				"mgt_demo -query_exposure_info [device_ip] for get current expsoure statistics information\n"
				"mgt_demo -set_exposure [device_ip] for set exposure attributes \n"
				"mgt_demo -switch_wdr [device_ip]  [flag] switch WDR mode with flag, 0 close WDR, 1 open WDR \n"
				"mgt_demo -rotate_video [device_ip] [r] rotate video with 0, 90, 180, 270 \n"
				"mgt_demo -set_luma [device_ip] [luma] set luma for color within 0 - 100 \n"
				"mgt_demo -set_rtctime [device_ip] [rtc time] set device rtc time, MMddhhmmyyyy.ss format"

		);
	}

	const char * device_ip = argv[2];

	if (argc == 5 && strcmp(argv[1], "-change_password") == 0) {
		auto ret = change_device_password(device_ip, argv[3], argv[4]);
		if (ret == API_STATUS::OK) {
			printf("change login password success!\n");
		}
		else {
			printf("change login password failed!\n");
		}
	}

	if (argc == 3 && strcmp(argv[1], "-reboot") == 0) {
		reboot_device(device_ip);
		printf("device will reboot in 30 seconds!\n");
	}

	if (argc == 3 && strcmp(argv[1], "-snapshot") == 0) {
		take_snapshot(device_ip);
	}

	if (argc == 3 && strcmp(argv[1], "-set_exposure") == 0 ) {
		// get ISP exposure attribute first
		// modify attributes
		// apply the attributes to device

		ISP_EXPOSURE_ATTR_S ispExpAttrs;
		auto ret = get_isp_exposure_attr(device_ip, &ispExpAttrs);
		if (ret == API_STATUS::OK) {
			ispExpAttrs.stAuto.stAGainRange.u32Min = 1280;
			ispExpAttrs.stAuto.stAGainRange.u32Max = 32000;

			ret = set_isp_exposure_attr(device_ip, &ispExpAttrs);
			if (ret == API_STATUS::OK) {
				printf("set ISP exposure attributes success!\n");
			}
			else {
				printf("set ISP exposure attributes failed!\n");
			}
		}
		else {
			printf("get ISP exposure attributes failed!\n");
		}

	}

	if (argc == 3 && strcmp(argv[1], "-get_anchorbox") == 0) {
			float xmin, ymin, xmax, ymax;
			if (get_anchor_box(device_ip, &xmin, &ymin, &xmax, &ymax) ==  API_STATUS::OK) {
				printf("get anchox xmin: %f, ymin: %f, xmax:%f, ymax:%f\n",
						xmin, ymin, xmax, ymax);
			}
			else {
				printf("get anchor box failed!\n");
			}
	 }

	if (argc == 7 && strcmp(argv[1], "-set_anchorbox") == 0) {
		float xmin = atof(argv[3]);
		float ymin = atof(argv[4]);
		float xmax = atof(argv[5]);
		float ymax = atof(argv[6]);
		set_anchor_box(device_ip, xmin, ymin, xmax, ymax);
	}

	if (argc==3 && strcmp(argv[1], "-deviceinfo") == 0) {
		DeviceInfo deviceInfo;
		if (get_device_info(device_ip, &deviceInfo) == API_STATUS::OK) {
			printf("device name: %s, IP: %s, firmware: %s, version: %s, minVersion: %d, productNo: %s, startup: %ld\n",
					deviceInfo.deviceName,
					deviceInfo.ip,
					deviceInfo.firmware,
					deviceInfo.version,
					deviceInfo.minVersion,
					deviceInfo.productNo,
					deviceInfo.startup
			);
		}
		else {
			printf("get device info error\n");
		}
	}

	if (argc==3 && strcmp(argv[1], "-query_exposure_info") == 0) {
		ISP_EXP_INFO_S expInfo;
		if (query_isp_exposure_info(device_ip, &expInfo) == API_STATUS::OK) {
			printf("Average Luma: %d, ISO: %d, exposure time: %d, AGain: %d, DGain: %d, ISPDGain: %d \n",
					expInfo.u8AveLum,  expInfo.u32ISO,
					expInfo.u32ExpTime, expInfo.u32AGain, expInfo.u32DGain, expInfo.u32ISPDGain);
		}
		else {
			printf("query exposure information failed!\n");
		}
	}

	if (argc == 4 && strcmp(argv[1], "-switch_wdr") == 0)
	{
		bool flag = atoi(argv[3]);
		switch_wdrmode(device_ip, flag);
		printf("switch WDR mode: %s \n", flag ? "ON":"OFF");

	}

	if (argc == 4 && strcmp(argv[1], "-rotate_video") == 0)
	{
		int r = atoi(argv[3]);
		rotate_video(device_ip, r);
		printf("rotate video %d \n", r);
	}

	if (argc == 4 && strcmp(argv[1], "-set_luma") == 0)
	{
		int luma = atoi(argv[3]);
		set_luma(device_ip, luma);
		printf("set color luma: %d", luma);

	}

	if (argc == 4 && strcmp(argv[1], "-set_rtctime") == 0)
	{
		set_rtctime(device_ip, argv[3], 15);
		printf("set rtc time: %s\n", argv[3]);
	}

}



